@@ -35,14 +35,15 @@
 #include <i915_drm.h>
 
 #include "drmtest.h"
+#include "igt_aux.h"
 #include "igt_kms.h"
 #include "igt_debugfs.h"
 
 /**
  * SECTION:igt_debugfs
  * @short_description: Support code for debugfs features
- * @title: i-g-t debugfs
- * @include: igt_debugfs.h
+ * @title: debugfs
+ * @include: igt.h
  *
  * This library provides helpers to access debugfs features. On top of some
  * basic functions to access debugfs files with e.g. igt_debugfs_open() it also
@@ -62,19 +63,19 @@
  * The only way to use #igt_crc_t CRCs therefore is to compare CRCs among each
  * another either for equality or difference. Otherwise CRCs must be treated as
  * completely opaque values. Note that not even CRCs from different pipes or tap
- * points on the same platform can be compared. Hence only use igt_crc_is_null()
- * and igt_crc_equal() to inspect CRC values captured by the same
+ * points on the same platform can be compared. Hence only use
+ * igt_assert_crc_equal() to inspect CRC values captured by the same
  * #igt_pipe_crc_t object.
  *
  * # Other debugfs interface wrappers
  *
  * This covers the miscellaneous debugfs interface wrappers:
  *
- * - drm/i915 supports interfaces to evict certain clases of gem buffer objects,
- *   see igt_drop_caches_set().
+ * - drm/i915 supports interfaces to evict certain classes of gem buffer
+ *   objects, see igt_drop_caches_set().
  *
  * - drm/i915 supports an interface to disable prefaulting, useful to test
- *   slowpaths in ioctls. See igt_disable_prefault().
+ *   slow paths in ioctls. See igt_disable_prefault().
  */
 
 /*
@@ -142,7 +143,7 @@ static igt_debugfs_t *__igt_debugfs_singleton(void)
  * @mode: mode bits as used by open()
  *
  * This opens a debugfs file as a Unix file descriptor. The filename should be
- * relative to the drm device's root, i.e without "drm/&lt;minor&gt;".
+ * relative to the drm device's root, i.e. without "drm/<minor>".
  *
  * Returns:
  * The Unix file descriptor for the debugfs file or -1 if that didn't work out.
@@ -165,7 +166,7 @@ int igt_debugfs_open(const char *filename, int mode)
  * @mode: mode string as used by fopen()
  *
  * This opens a debugfs file as a libc FILE. The filename should be
- * relative to the drm device's root, i.e without "drm/&lt;minor&gt;".
+ * relative to the drm device's root, i.e. without "drm/<minor>".
  *
  * Returns:
  * The libc FILE pointer for the debugfs file or NULL if that didn't work out.
@@ -184,54 +185,85 @@ FILE *igt_debugfs_fopen(const char *filename,
 	return fopen(buf, mode);
 }
 
+/**
+ * __igt_debugfs_read:
+ * @filename: file name
+ * @buf: buffer where the contents will be stored, allocated by the caller
+ * @buf_size: size of the buffer
+ *
+ * This function opens the debugfs file, reads it, stores the content in the
+ * provided buffer, then closes the file. Users should make sure that the buffer
+ * provided is big enough to fit the whole file, plus one byte.
+ */
+void __igt_debugfs_read(const char *filename, char *buf, int buf_size)
+{
+	FILE *file;
+	size_t n_read;
+
+	file = igt_debugfs_fopen(filename, "r");
+	igt_assert(file);
+
+	n_read = fread(buf, 1, buf_size - 1, file);
+	igt_assert(n_read > 0);
+	igt_assert(feof(file));
+
+	buf[n_read] = '\0';
+
+	igt_assert(fclose(file) == 0);
+}
+
+/**
+ * igt_debugfs_search:
+ * @filename: file name
+ * @substring: string to search for in @filename
+ *
+ * Searches each line in @filename for the substring specified in @substring.
+ *
+ * Returns: True if the @substring is found to occur in @filename
+ */
+bool igt_debugfs_search(const char *filename, const char *substring)
+{
+	FILE *file;
+	size_t n = 0;
+	char *line = NULL;
+	bool matched = false;
+
+	file = igt_debugfs_fopen(filename, "r");
+	igt_assert(file);
+
+	while (getline(&line, &n, file) >= 0) {
+		matched = (strstr(line, substring) != NULL);
+		if (matched)
+			break;
+	}
+
+	free(line);
+	fclose(file);
+
+	return matched;
+}
+
 /*
  * Pipe CRC
  */
 
 /**
- * igt_crc_is_null:
- * @crc: pipe CRC value to check
- *
- * Returns: True if the CRC is null/invalid, false if it represents a captured
- * valid CRC.
- */
-bool igt_crc_is_null(igt_crc_t *crc)
-{
-	int i;
-
-	for (i = 0; i < crc->n_words; i++) {
-		igt_warn_on_f(crc->crc[i] == 0xffffffff,
-			      "Suspicious CRC: it looks like the CRC "
-			      "read back was from a register in a powered "
-			      "down well\n");
-		if (crc->crc[i])
-			return false;
-	}
-
-	return true;
-}
-
-/**
- * igt_crc_equal:
+ * igt_assert_crc_equal:
  * @a: first pipe CRC value
  * @b: second pipe CRC value
  *
- * Compares two CRC values.
- *
- * Returns: true if the two CRCs match, false otherwise.
+ * Compares two CRC values and fails the testcase if they don't match with
+ * igt_fail(). Note that due to CRC collisions CRC based testcase can only
+ * assert that CRCs match, never that they are different. Otherwise there might
+ * be random testcase failures when different screen contents end up with the
+ * same CRC by chance.
  */
-bool igt_crc_equal(igt_crc_t *a, igt_crc_t *b)
+void igt_assert_crc_equal(igt_crc_t *a, igt_crc_t *b)
 {
 	int i;
 
-	if (a->n_words != b->n_words)
-		return false;
-
 	for (i = 0; i < a->n_words; i++)
-		if (a->crc[i] != b->crc[i])
-			return false;
-
-	return true;
+		igt_assert_eq_u32(a->crc[i], b->crc[i]);
 }
 
 /**
@@ -300,7 +332,7 @@ static bool igt_pipe_crc_do_start(igt_pipe_crc_t *pipe_crc)
 	sprintf(buf, "pipe %s %s", kmstest_pipe_name(pipe_crc->pipe),
 		pipe_crc_source_name(pipe_crc->source));
 	errno = 0;
-	write(pipe_crc->ctl_fd, buf, strlen(buf));
+	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
 	if (errno != 0)
 		return false;
 
@@ -312,7 +344,7 @@ static void igt_pipe_crc_pipe_off(int fd, enum pipe pipe)
 	char buf[32];
 
 	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe));
-	write(fd, buf, strlen(buf));
+	igt_assert_eq(write(fd, buf, strlen(buf)), strlen(buf));
 }
 
 static void igt_pipe_crc_reset(void)
@@ -444,7 +476,7 @@ void igt_pipe_crc_stop(igt_pipe_crc_t *pipe_crc)
 	char buf[32];
 
 	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe_crc->pipe));
-	write(pipe_crc->ctl_fd, buf, strlen(buf));
+	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
 }
 
 static bool pipe_crc_init_from_string(igt_crc_t *crc, const char *line)
@@ -462,9 +494,9 @@ static bool read_one_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 	ssize_t bytes_read;
 	char buf[pipe_crc->buffer_len];
 
-	igt_set_timeout(5);
+	igt_set_timeout(5, "CRC reading");
 	bytes_read = read(pipe_crc->crc_fd, &buf, pipe_crc->line_len);
-	igt_set_timeout(0);
+	igt_reset_timeout();
 
 	igt_assert_eq(bytes_read, pipe_crc->line_len);
 	buf[bytes_read] = '\0';
@@ -509,6 +541,23 @@ igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
 	*out_crcs = crcs;
 }
 
+static void crc_sanity_checks(igt_crc_t *crc)
+{
+	int i;
+	bool all_zero = true;
+
+	for (i = 0; i < crc->n_words; i++) {
+		igt_warn_on_f(crc->crc[i] == 0xffffffff,
+			      "Suspicious CRC: it looks like the CRC "
+			      "read back was from a register in a powered "
+			      "down well\n");
+		if (crc->crc[i])
+			all_zero = false;
+	}
+
+	igt_warn_on_f(all_zero, "Suspicious CRC: All values are 0.\n");
+}
+
 /**
  * igt_pipe_crc_collect_crc:
  * @pipe_crc: pipe CRC object
@@ -519,14 +568,19 @@ igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
  *
  * This function takes care of the pipe_crc book-keeping, it will start/stop
  * the collection of the CRC.
+ *
+ * This function also calls the interactive debug with the "crc" domain, so you
+ * can make use of this feature to actually see the screen that is being CRC'd.
  */
 void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
 {
+	igt_debug_wait_for_keypress("crc");
+
 	igt_pipe_crc_start(pipe_crc);
 	read_one_crc(pipe_crc, out_crc);
 	igt_pipe_crc_stop(pipe_crc);
 
-	igt_assert(!igt_crc_is_null(out_crc));
+	crc_sanity_checks(out_crc);
 }
 
 /*
