@@ -36,12 +36,12 @@
 /**
  * SECTION:igt_fb
  * @short_description: Framebuffer handling and drawing library
- * @title: i-g-t framebuffer
- * @include: igt_fb.h
+ * @title: Framebuffer
+ * @include: igt.h
  *
  * This library contains helper functions for handling kms framebuffer objects
  * using #igt_fb structures to track all the metadata.  igt_create_fb() creates
- * a basic framebufffer and igt_remove_fb() cleans everything up again.
+ * a basic framebuffer and igt_remove_fb() cleans everything up again.
  *
  * It also supports drawing using the cairo library and provides some simplified
  * helper functions to easily draw test patterns. The main function to create a
@@ -76,9 +76,8 @@ static struct format_desc_struct {
 /* helpers to create nice-looking framebuffers */
 static int create_bo_for_fb(int fd, int width, int height, int bpp,
 			    uint64_t tiling, unsigned bo_size,
-			    uint32_t *gem_handle_ret,
-			    unsigned *size_ret,
-			    unsigned *stride_ret)
+			    unsigned bo_stride, uint32_t *gem_handle_ret,
+			    unsigned *size_ret, unsigned *stride_ret)
 {
 	uint32_t gem_handle;
 	int size, ret = 0;
@@ -104,19 +103,23 @@ static int create_bo_for_fb(int fd, int width, int height, int bpp,
 			;
 	} else {
 		/* Scan-out has a 64 byte alignment restriction */
-		stride = (width * (bpp / 8) + 63) & ~63;
+		stride = ALIGN(width * (bpp / 8), 64);
 		size = stride * height;
 	}
 
 	if (bo_size == 0)
 		bo_size = size;
+	if (bo_stride == 0)
+		bo_stride = stride;
+
 	gem_handle = gem_create(fd, bo_size);
 
 	if (tiling == LOCAL_I915_FORMAT_MOD_X_TILED)
-		ret = __gem_set_tiling(fd, gem_handle, I915_TILING_X, stride);
+		ret = __gem_set_tiling(fd, gem_handle, I915_TILING_X,
+				       bo_stride);
 
-	*stride_ret = stride;
-	*size_ret = size;
+	*stride_ret = bo_stride;
+	*size_ret = bo_size;
 	*gem_handle_ret = gem_handle;
 
 	return ret;
@@ -130,7 +133,7 @@ static int create_bo_for_fb(int fd, int width, int height, int bpp,
  * @w: width of the fill rectangle
  * @h: height of the fill rectangle
  * @r: red value to use as fill color
- * @g: gree value to use as fill color
+ * @g: green value to use as fill color
  * @b: blue value to use as fill color
  *
  * This functions draws a solid rectangle with the given color using the drawing
@@ -152,7 +155,7 @@ void igt_paint_color(cairo_t *cr, int x, int y, int w, int h,
  * @w: width of the fill rectangle
  * @h: height of the fill rectangle
  * @r: red value to use as fill color
- * @g: gree value to use as fill color
+ * @g: green value to use as fill color
  * @b: blue value to use as fill color
  * @a: alpha value to use as fill color
  *
@@ -175,7 +178,7 @@ void igt_paint_color_alpha(cairo_t *cr, int x, int y, int w, int h,
  * @w: width of the fill rectangle
  * @h: height of the fill rectangle
  * @r: red value to use as fill color
- * @g: gree value to use as fill color
+ * @g: green value to use as fill color
  * @b: blue value to use as fill color
  *
  * This functions draws a gradient into the rectangle which fades in from black
@@ -338,6 +341,15 @@ void igt_paint_test_pattern(cairo_t *cr, int width, int height)
 	igt_assert(!cairo_status(cr));
 }
 
+static cairo_status_t
+stdio_read_func(void *closure, unsigned char* data, unsigned int size)
+{
+	if (fread(data, 1, size, (FILE*)closure) != size)
+		return CAIRO_STATUS_READ_ERROR;
+
+	return CAIRO_STATUS_SUCCESS;
+}
+
 /**
  * igt_paint_image:
  * @cr: cairo drawing context
@@ -347,9 +359,8 @@ void igt_paint_test_pattern(cairo_t *cr, int width, int height)
  * @dst_width: width of the destination rectangle
  * @dst_height: height of the destination rectangle
  *
- * This function can be used to draw a scaled version of the supplied png image.
- * This is currently only used by the CR-code based testing in the "testdisplay"
- * testcase.
+ * This function can be used to draw a scaled version of the supplied png image,
+ * which is loaded from the package data directory.
  */
 void igt_paint_image(cairo_t *cr, const char *filename,
 		     int dst_x, int dst_y, int dst_width, int dst_height)
@@ -357,8 +368,11 @@ void igt_paint_image(cairo_t *cr, const char *filename,
 	cairo_surface_t *image;
 	int img_width, img_height;
 	double scale_x, scale_y;
+	FILE* f;
 
-	image = cairo_image_surface_create_from_png(filename);
+	f = igt_fopen_data(filename);
+
+	image = cairo_image_surface_create_from_png_stream(&stdio_read_func, f);
 	igt_assert(cairo_surface_status(image) == CAIRO_STATUS_SUCCESS);
 
 	img_width = cairo_image_surface_get_width(image);
@@ -377,6 +391,8 @@ void igt_paint_image(cairo_t *cr, const char *filename,
 	cairo_surface_destroy(image);
 
 	cairo_restore(cr);
+
+	fclose(f);
 }
 
 /**
@@ -387,7 +403,8 @@ void igt_paint_image(cairo_t *cr, const char *filename,
  * @format: drm fourcc pixel format code
  * @tiling: tiling layout of the framebuffer (as framebuffer modifier)
  * @fb: pointer to an #igt_fb structure
- * @bo_size: size of the backing bo (0 for minimum needed size)
+ * @bo_size: size of the backing bo (0 for automatic size)
+ * @bo_stride: stride of the backing bo (0 for automatic stride)
  *
  * This function allocates a gem buffer object suitable to back a framebuffer
  * with the requested properties and then wraps it up in a drm framebuffer
@@ -402,7 +419,8 @@ void igt_paint_image(cairo_t *cr, const char *filename,
 unsigned int
 igt_create_fb_with_bo_size(int fd, int width, int height,
 			   uint32_t format, uint64_t tiling,
-			   struct igt_fb *fb, unsigned bo_size)
+			   struct igt_fb *fb, unsigned bo_size,
+			   unsigned bo_stride)
 {
 	uint32_t fb_id;
 	int bpp;
@@ -411,10 +429,11 @@ igt_create_fb_with_bo_size(int fd, int width, int height,
 
 	bpp = igt_drm_format_to_bpp(format);
 
-	igt_debug("%s(width=%d, height=%d, format=0x%x [bpp=%d], tiling=0x%"PRIx64", size=%d\n",
+	igt_debug("%s(width=%d, height=%d, format=0x%x [bpp=%d], tiling=0x%"PRIx64", size=%d)\n",
 		  __func__, width, height, format, bpp, tiling, bo_size);
 	do_or_die(create_bo_for_fb(fd, width, height, bpp, tiling, bo_size,
-				   &fb->gem_handle, &fb->size, &fb->stride));
+				   bo_stride, &fb->gem_handle, &fb->size,
+				   &fb->stride));
 
 	igt_debug("%s(handle=%d, pitch=%d)\n",
 		  __func__, fb->gem_handle, fb->stride);
@@ -472,7 +491,8 @@ igt_create_fb_with_bo_size(int fd, int width, int height,
 unsigned int igt_create_fb(int fd, int width, int height, uint32_t format,
 			   uint64_t tiling, struct igt_fb *fb)
 {
-	return igt_create_fb_with_bo_size(fd, width, height, format, tiling, fb, 0);
+	return igt_create_fb_with_bo_size(fd, width, height, format, tiling, fb,
+					  0, 0);
 }
 
 /**
@@ -605,10 +625,10 @@ unsigned int igt_create_stereo_fb(int drm_fd, drmModeModeInfo *mode,
 			      tiling, &fb);
 	cr = igt_get_cairo_ctx(drm_fd, &fb);
 
-	igt_paint_image(cr, IGT_DATADIR"/1080p-left.png",
+	igt_paint_image(cr, "1080p-left.png",
 			layout.left.x, layout.left.y,
 			layout.left.width, layout.left.height);
-	igt_paint_image(cr, IGT_DATADIR"/1080p-right.png",
+	igt_paint_image(cr, "1080p-right.png",
 			layout.right.x, layout.right.y,
 			layout.right.width, layout.right.height);
 
@@ -639,11 +659,27 @@ struct fb_blit_upload {
 	} linear;
 };
 
+static unsigned int fb_mod_to_obj_tiling(uint64_t fb_mod)
+{
+	switch (fb_mod) {
+	case LOCAL_DRM_FORMAT_MOD_NONE:
+		return I915_TILING_NONE;
+	case LOCAL_I915_FORMAT_MOD_X_TILED:
+		return I915_TILING_X;
+	case LOCAL_I915_FORMAT_MOD_Y_TILED:
+		return I915_TILING_Y;
+	case LOCAL_I915_FORMAT_MOD_Yf_TILED:
+		return I915_TILING_Yf;
+	default:
+		igt_assert(0);
+	}
+}
+
 static void destroy_cairo_surface__blit(void *arg)
 {
 	struct fb_blit_upload *blit = arg;
 	struct igt_fb *fb = blit->fb;
-	unsigned int obj_tiling = I915_TILING_NONE;
+	unsigned int obj_tiling = fb_mod_to_obj_tiling(fb->tiling);
 
 	munmap(blit->linear.map, blit->linear.size);
 	fb->cairo_surface = NULL;
@@ -651,18 +687,6 @@ static void destroy_cairo_surface__blit(void *arg)
 	gem_set_domain(blit->fd, blit->linear.handle,
 			I915_GEM_DOMAIN_GTT, 0);
 
-	switch (fb->tiling) {
-	case LOCAL_I915_FORMAT_MOD_X_TILED:
-		obj_tiling = I915_TILING_X;
-		break;
-	case LOCAL_I915_FORMAT_MOD_Y_TILED:
-		obj_tiling = I915_TILING_Y;
-		break;
-	case LOCAL_I915_FORMAT_MOD_Yf_TILED:
-		obj_tiling = I915_TILING_Yf;
-		break;
-	}
-
 	igt_blitter_fast_copy__raw(blit->fd,
 				   blit->linear.handle,
 				   blit->linear.stride,
@@ -684,6 +708,7 @@ static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
 {
 	struct fb_blit_upload *blit;
 	cairo_format_t cairo_format;
+	unsigned int obj_tiling = fb_mod_to_obj_tiling(fb->tiling);
 	int bpp, ret;
 
 	blit = malloc(sizeof(*blit));
@@ -696,7 +721,7 @@ static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
 	 */
 	bpp = igt_drm_format_to_bpp(fb->drm_format);
 	ret = create_bo_for_fb(fd, fb->width, fb->height, bpp,
-				LOCAL_DRM_FORMAT_MOD_NONE, 0,
+				LOCAL_DRM_FORMAT_MOD_NONE, 0, 0,
 				&blit->linear.handle,
 				&blit->linear.size,
 				&blit->linear.stride);
@@ -705,15 +730,33 @@ static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
 
 	blit->fd = fd;
 	blit->fb = fb;
+
+	/* Copy fb content to linear BO */
+	gem_set_domain(fd, blit->linear.handle,
+			I915_GEM_DOMAIN_GTT, 0);
+
+	igt_blitter_fast_copy__raw(fd,
+				   fb->gem_handle,
+				   fb->stride,
+				   obj_tiling,
+				   0, 0, /* src_x, src_y */
+				   fb->width, fb->height,
+				   blit->linear.handle,
+				   blit->linear.stride,
+				   I915_TILING_NONE,
+				   0, 0 /* dst_x, dst_y */);
+
+	gem_sync(fd, blit->linear.handle);
+
+	gem_set_domain(fd, blit->linear.handle,
+		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+
+	/* Setup cairo context */
 	blit->linear.map = gem_mmap__cpu(fd,
 					 blit->linear.handle,
 					 0,
 					 blit->linear.size,
 					 PROT_READ | PROT_WRITE);
-	igt_assert(blit->linear.map);
-
-	gem_set_domain(fd, blit->linear.handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
 
 	cairo_format = drm_format_to_cairo(fb->drm_format);
 	fb->cairo_surface =
@@ -737,8 +780,10 @@ static void destroy_cairo_surface__gtt(void *arg)
 
 static void create_cairo_surface__gtt(int fd, struct igt_fb *fb)
 {
+	void *ptr = gem_mmap__gtt(fd, fb->gem_handle, fb->size, PROT_READ | PROT_WRITE);
+
 	fb->cairo_surface =
-		cairo_image_surface_create_for_data(gem_mmap(fd, fb->gem_handle, fb->size, PROT_READ | PROT_WRITE),
+		cairo_image_surface_create_for_data(ptr,
 						    drm_format_to_cairo(fb->drm_format),
 						    fb->width, fb->height, fb->stride);
 
