@@ -62,7 +62,7 @@
  * SECTION:ioctl_wrappers
  * @short_description: ioctl wrappers and related functions
  * @title: ioctl wrappers
- * @include: ioctl_wrappers.h
+ * @include: igt.h
  *
  * This helper library contains simple functions to wrap the raw drm/i915 kernel
  * ioctls. The normal versions never pass any error codes to the caller and use
@@ -199,9 +199,10 @@ void gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 	memset(&arg, 0, sizeof(arg));
 	arg.handle = handle;
 	arg.caching = caching;
-	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
 
+	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
 	igt_assert(ret == 0 || (errno == ENOTTY || errno == EINVAL));
+
 	igt_require(ret == 0);
 	errno = 0;
 }
@@ -306,7 +307,7 @@ void gem_close(int fd, uint32_t handle)
  * This wraps the PWRITE ioctl, which is to upload a linear data to a subrange
  * of a gem buffer object.
  */
-void gem_write(int fd, uint32_t handle, uint32_t offset, const void *buf, uint32_t length)
+void gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
 {
 	struct drm_i915_gem_pwrite gem_pwrite;
 
@@ -329,7 +330,7 @@ void gem_write(int fd, uint32_t handle, uint32_t offset, const void *buf, uint32
  * This wraps the PREAD ioctl, which is to download a linear data to a subrange
  * of a gem buffer object.
  */
-void gem_read(int fd, uint32_t handle, uint32_t offset, void *buf, uint32_t length)
+void gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t length)
 {
 	struct drm_i915_gem_pread gem_pread;
 
@@ -371,12 +372,22 @@ void gem_set_domain(int fd, uint32_t handle,
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  *
- * This is a wrapper around gem_set_domain() which simply blocks for any
- * outstanding rendering to complete.
+ * This functions waits for outstanding rendering to complete.
  */
 void gem_sync(int fd, uint32_t handle)
 {
-	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	struct drm_i915_gem_wait wait;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.bo_handle = handle;
+	wait.timeout_ns =-1;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait) == 0) {
+		errno = 0;
+		return;
+	}
+
+	gem_set_domain(fd, handle,
+		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 }
 
 uint32_t __gem_create(int fd, int size)
@@ -406,7 +417,7 @@ uint32_t __gem_create(int fd, int size)
  *
  * Returns: The file-private handle of the created buffer object
  */
-uint32_t gem_create(int fd, int size)
+uint32_t gem_create(int fd, uint64_t size)
 {
 	struct drm_i915_gem_create create;
 
@@ -429,17 +440,15 @@ uint32_t gem_create(int fd, int size)
  */
 void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
-	int ret;
+	int result;
 
-	ret = drmIoctl(fd,
-		       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-		       execbuf);
-	igt_assert(ret == 0);
+	result = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf);
+	igt_assert(result == 0);
 	errno = 0;
 }
 
 /**
- * gem_mmap__gtt:
+ * __gem_mmap__gtt:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  * @size: size of the gem buffer
@@ -448,9 +457,9 @@ void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
  * This functions wraps up procedure to establish a memory mapping through the
  * GTT.
  *
- * Returns: A pointer to the created memory mapping.
+ * Returns: A pointer to the created memory mapping, NULL on failure.
  */
-void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
+void *__gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 {
 	struct drm_i915_gem_mmap_gtt mmap_arg;
 	void *ptr;
@@ -469,6 +478,24 @@ void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
 	return ptr;
 }
 
+/**
+ * gem_mmap__gtt:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @size: size of the gem buffer
+ * @prot: memory protection bits as used by mmap()
+ *
+ * Like __gem_mmap__gtt() except we assert on failure.
+ *
+ * Returns: A pointer to the created memory mapping
+ */
+void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
+{
+	void *ptr = __gem_mmap__gtt(fd, handle, size, prot);
+	igt_assert(ptr);
+	return ptr;
+}
+
 struct local_i915_gem_mmap_v2 {
 	uint32_t handle;
 	uint32_t pad;
@@ -515,10 +542,10 @@ bool gem_mmap__has_wc(int fd)
 }
 
 /**
- * gem_mmap__wc:
+ * __gem_mmap__wc:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
- * @offset: offset in the gem buffer of te mmap arena
+ * @offset: offset in the gem buffer of the mmap arena
  * @size: size of the mmap arena
  * @prot: memory protection bits as used by mmap()
  *
@@ -527,9 +554,9 @@ bool gem_mmap__has_wc(int fd)
  * bypassing the GTT system agent (i.e. there is no automatic tiling of
  * the mmapping through the fence registers).
  *
- * Returns: A pointer to the created memory mapping.
+ * Returns: A pointer to the created memory mapping, NULL on failure.
  */
-void *gem_mmap__wc(int fd, uint32_t handle, int offset, int size, int prot)
+void *__gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
 {
 	struct local_i915_gem_mmap_v2 arg;
 
@@ -551,19 +578,38 @@ void *gem_mmap__wc(int fd, uint32_t handle, int offset, int size, int prot)
 }
 
 /**
- * gem_mmap__cpu:
+ * gem_mmap__wc:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
- * @offset: offset in the gem buffer of te mmap arena
+ * @offset: offset in the gem buffer of the mmap arena
+ * @size: size of the mmap arena
+ * @prot: memory protection bits as used by mmap()
+ *
+ * Like __gem_mmap__wc() except we assert on failure.
+ *
+ * Returns: A pointer to the created memory mapping
+ */
+void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+{
+	void *ptr = __gem_mmap__wc(fd, handle, offset, size, prot);
+	igt_assert(ptr);
+	return ptr;
+}
+
+/**
+ * __gem_mmap__cpu:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @offset: offset in the gem buffer of the mmap arena
  * @size: size of the mmap arena
  * @prot: memory protection bits as used by mmap()
  *
  * This functions wraps up procedure to establish a memory mapping through
  * direct cpu access, bypassing the gpu completely.
  *
- * Returns: A pointer to the created memory mapping.
+ * Returns: A pointer to the created memory mapping, NULL on failure.
  */
-void *gem_mmap__cpu(int fd, uint32_t handle, int offset, int size, int prot)
+void *__gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
 {
 	struct drm_i915_gem_mmap mmap_arg;
 
@@ -578,6 +624,25 @@ void *gem_mmap__cpu(int fd, uint32_t handle, int offset, int size, int prot)
 	return (void *)(uintptr_t)mmap_arg.addr_ptr;
 }
 
+/**
+ * gem_mmap__cpu:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @offset: offset in the gem buffer of the mmap arena
+ * @size: size of the mmap arena
+ * @prot: memory protection bits as used by mmap()
+ *
+ * Like __gem_mmap__cpu() except we assert on failure.
+ *
+ * Returns: A pointer to the created memory mapping
+ */
+void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+{
+	void *ptr = __gem_mmap__cpu(fd, handle, offset, size, prot);
+	igt_assert(ptr);
+	return ptr;
+}
+
 /**
  * gem_madvise:
  * @fd: open i915 drm file descriptor
@@ -716,6 +781,24 @@ void gem_context_require_param(int fd, uint64_t param)
 	igt_require(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
 }
 
+void gem_context_require_ban_period(int fd)
+{
+	static int has_ban_period = -1;
+
+	if (has_ban_period < 0) {
+		struct local_i915_gem_context_param p;
+
+		p.context = 0;
+		p.param = LOCAL_CONTEXT_PARAM_BAN_PERIOD;
+		p.value = 0;
+		p.size = 0;
+
+		has_ban_period = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
+	}
+
+	igt_require(has_ban_period);
+}
+
 /**
  * gem_sw_finish:
  * @fd: open i915 drm file descriptor
@@ -799,34 +882,40 @@ bool gem_uses_aliasing_ppgtt(int fd)
  */
 int gem_available_fences(int fd)
 {
-	struct drm_i915_getparam gp;
-	int val = 0;
+	static int num_fences = -1;
 
-	memset(&gp, 0, sizeof(gp));
-	gp.param = I915_PARAM_NUM_FENCES_AVAIL;
-	gp.value = &val;
+	if (num_fences < 0) {
+		struct drm_i915_getparam gp;
 
-	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
-		return 0;
+		memset(&gp, 0, sizeof(gp));
+		gp.param = I915_PARAM_NUM_FENCES_AVAIL;
+		gp.value = &num_fences;
 
-	errno = 0;
-	return val;
+		num_fences = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return num_fences;
 }
 
 bool gem_has_llc(int fd)
 {
-	struct drm_i915_getparam gp;
-	int val = 0;
+	static int has_llc = -1;
 
-	memset(&gp, 0, sizeof(gp));
-	gp.param = I915_PARAM_HAS_LLC;
-	gp.value = &val;
+	if (has_llc < 0) {
+		struct drm_i915_getparam gp;
 
-	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
-		return 0;
+		memset(&gp, 0, sizeof(gp));
+		gp.param = I915_PARAM_HAS_LLC;
+		gp.value = &has_llc;
 
-	errno = 0;
-	return val;
+		has_llc = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return has_llc;
 }
 
 /**
@@ -843,24 +932,26 @@ bool gem_has_llc(int fd)
  */
 int gem_get_num_rings(int fd)
 {
-	int num_rings = 1;	/* render ring is always available */
+	static int num_rings = -1;
 
-	if (gem_has_bsd(fd))
-		num_rings++;
-	else
-		goto skip;
+	if (num_rings < 0) {
+		num_rings = 1;	/* render ring is always available */
 
-	if (gem_has_blt(fd))
-		num_rings++;
-	else
-		goto skip;
-
-	if (gem_has_vebox(fd))
-		num_rings++;
-	else
-		goto skip;
+		if (gem_has_bsd(fd))
+			num_rings++;
+		else
+			goto skip;
 
+		if (gem_has_blt(fd))
+			num_rings++;
+		else
+			goto skip;
 
+		if (gem_has_vebox(fd))
+			num_rings++;
+		else
+			goto skip;
+	}
 skip:
 	return num_rings;
 }
@@ -903,7 +994,10 @@ bool gem_has_enable_ring(int fd,int param)
  */
 bool gem_has_bsd(int fd)
 {
-	return gem_has_enable_ring(fd,I915_PARAM_HAS_BSD);
+	static int has_bsd = -1;
+	if (has_bsd < 0)
+		has_bsd = gem_has_enable_ring(fd,I915_PARAM_HAS_BSD);
+	return has_bsd;
 }
 
 /**
@@ -919,7 +1013,10 @@ bool gem_has_bsd(int fd)
  */
 bool gem_has_blt(int fd)
 {
-	return gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
+	static int has_blt = -1;
+	if (has_blt < 0)
+		has_blt =  gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
+	return has_blt;
 }
 
 #define LOCAL_I915_PARAM_HAS_VEBOX 22
@@ -937,7 +1034,10 @@ bool gem_has_blt(int fd)
  */
 bool gem_has_vebox(int fd)
 {
-	return gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
+	static int has_vebox = -1;
+	if (has_vebox < 0)
+		has_vebox =  gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
+	return has_vebox;
 }
 
 #define LOCAL_I915_PARAM_HAS_BSD2 31
@@ -954,7 +1054,10 @@ bool gem_has_vebox(int fd)
  */
 bool gem_has_bsd2(int fd)
 {
-	return gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_BSD2);
+	static int has_bsd2 = -1;
+	if (has_bsd2 < 0)
+		has_bsd2 = gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_BSD2);
+	return has_bsd2;
 }
 /**
  * gem_available_aperture_size:
@@ -986,13 +1089,27 @@ uint64_t gem_available_aperture_size(int fd)
  */
 uint64_t gem_aperture_size(int fd)
 {
-	struct drm_i915_gem_get_aperture aperture;
+	static uint64_t aperture_size = 0;
 
-	memset(&aperture, 0, sizeof(aperture));
-	aperture.aper_size = 256*1024*1024;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+	if (aperture_size == 0) {
+		struct local_i915_gem_context_param p;
 
-	return aperture.aper_size;
+		memset(&p, 0, sizeof(p));
+		p.param = 0x3;
+		if (ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0) {
+			aperture_size = p.value;
+		} else {
+			struct drm_i915_gem_get_aperture aperture;
+
+			memset(&aperture, 0, sizeof(aperture));
+			aperture.aper_size = 256*1024*1024;
+
+			do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+			aperture_size =  aperture.aper_size;
+		}
+	}
+
+	return aperture_size;
 }
 
 /**
@@ -1055,10 +1172,10 @@ void gem_require_ring(int fd, int ring_id)
 	case I915_EXEC_RENDER:
 		return;
 	case I915_EXEC_BLT:
-		igt_require(HAS_BLT_RING(intel_get_drm_devid(fd)));
+		igt_require(gem_has_blt(fd));
 		return;
 	case I915_EXEC_BSD:
-		igt_require(HAS_BSD_RING(intel_get_drm_devid(fd)));
+		igt_require(gem_has_bsd(fd));
 		return;
 #ifdef I915_EXEC_VEBOX
 	case I915_EXEC_VEBOX:
