@@ -106,6 +106,20 @@ unmapped_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
 	return bo;
 }
 
+static drm_intel_bo *
+snoop_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
+{
+	drm_intel_bo *bo;
+
+	igt_skip_on(gem_has_llc(fd));
+
+	bo = unmapped_create_bo(bufmgr, width, height);
+	gem_set_caching(fd, bo->handle, I915_CACHING_CACHED);
+	drm_intel_bo_disable_reuse(bo);
+
+	return bo;
+}
+
 static void
 gtt_set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 {
@@ -171,7 +185,7 @@ wc_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
 	gem_require_mmap_wc(fd);
 
 	bo = unmapped_create_bo(bufmgr, width, height);
-	bo->virtual = gem_mmap__wc(fd, bo->handle, 0, bo->size, PROT_READ | PROT_WRITE);
+	bo->virtual = __gem_mmap__wc(fd, bo->handle, 0, bo->size, PROT_READ | PROT_WRITE);
 	return bo;
 }
 
@@ -284,8 +298,8 @@ gpu_set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 	gem_pwrite.offset = 0;
 	gem_pwrite.size = execbuf.batch_len;
 	gem_pwrite.data_ptr = (uintptr_t)buf;
-	do_or_die(drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite));
-	do_or_die(drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf));
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite);
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf);
 
 	drmIoctl(fd, DRM_IOCTL_GEM_CLOSE, &create.handle);
 }
@@ -321,6 +335,13 @@ const struct access_mode {
 		.create_bo = unmapped_create_bo,
 		.release_bo = nop_release_bo,
 	},
+	{
+		.name = "snoop",
+		.set_bo = cpu_set_bo,
+		.cmp_bo = cpu_cmp_bo,
+		.create_bo = snoop_create_bo,
+		.release_bo = nop_release_bo,
+	},
 	{
 		.name = "gtt",
 		.set_bo = gtt_set_bo,
@@ -472,9 +493,7 @@ static void cpu_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
 	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_CPU, 0);
 	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
 	s = gem_mmap__cpu(fd, src->handle, 0, size, PROT_READ);
-	igt_assert(s != NULL);
 	d = gem_mmap__cpu(fd, dst->handle, 0, size, PROT_WRITE);
-	igt_assert(d != NULL);
 
 	memcpy(d, s, size);
 
@@ -491,9 +510,7 @@ static void gtt_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
 	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 
 	s = gem_mmap__gtt(fd, src->handle, size, PROT_READ);
-	igt_assert(s != NULL);
 	d = gem_mmap__gtt(fd, dst->handle, size, PROT_WRITE);
-	igt_assert(d != NULL);
 
 	memcpy(d, s, size);
 
@@ -510,9 +527,7 @@ static void wc_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
 	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 
 	s = gem_mmap__wc(fd, src->handle, 0, size, PROT_READ);
-	igt_assert(s != NULL);
 	d = gem_mmap__wc(fd, dst->handle, 0, size, PROT_WRITE);
-	igt_assert(d != NULL);
 
 	memcpy(d, s, size);
 
@@ -741,6 +756,26 @@ static void do_read_read_bcs(struct buffers *buffers,
 	igt_post_hang_ring(fd, hang);
 }
 
+static void do_write_read_bcs(struct buffers *buffers,
+			      do_copy do_copy_func,
+			      do_hang do_hang_func)
+{
+	struct igt_hang_ring hang;
+	int i;
+
+	gem_quiescent_gpu(fd);
+	for (i = buffers->count; i--; )
+		buffers->mode->set_bo(buffers->src[i], 0xdeadbeef ^ i, width, height);
+	for (i = 0; i < buffers->count; i++) {
+		blt_copy_bo(buffers->spare, buffers->src[i]);
+		do_copy_func(buffers->dst[i], buffers->spare);
+	}
+	hang = do_hang_func();
+	for (i = buffers->count; i--; )
+		buffers->mode->cmp_bo(buffers->dst[i], 0xdeadbeef ^ i, width, height, buffers->dummy);
+	igt_post_hang_ring(fd, hang);
+}
+
 static void do_read_read_rcs(struct buffers *buffers,
 			     do_copy do_copy_func,
 			     do_hang do_hang_func)
@@ -762,6 +797,26 @@ static void do_read_read_rcs(struct buffers *buffers,
 	igt_post_hang_ring(fd, hang);
 }
 
+static void do_write_read_rcs(struct buffers *buffers,
+			      do_copy do_copy_func,
+			      do_hang do_hang_func)
+{
+	struct igt_hang_ring hang;
+	int i;
+
+	gem_quiescent_gpu(fd);
+	for (i = buffers->count; i--; )
+		buffers->mode->set_bo(buffers->src[i], 0xdeadbeef ^ i, width, height);
+	for (i = 0; i < buffers->count; i++) {
+		render_copy_bo(buffers->spare, buffers->src[i]);
+		do_copy_func(buffers->dst[i], buffers->spare);
+	}
+	hang = do_hang_func();
+	for (i = buffers->count; i--; )
+		buffers->mode->cmp_bo(buffers->dst[i], 0xdeadbeef ^ i, width, height, buffers->dummy);
+	igt_post_hang_ring(fd, hang);
+}
+
 static void do_gpu_read_after_write(struct buffers *buffers,
 				    do_copy do_copy_func,
 				    do_hang do_hang_func)
@@ -853,7 +908,7 @@ static void bit17_require(void)
 	arg.handle = gem_create(fd, 4096);
 	gem_set_tiling(fd, arg.handle, I915_TILING_X, 512);
 
-	do_or_die(drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_TILING2, &arg));
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_TILING2, &arg);
 	gem_close(fd, arg.handle);
 	igt_require(arg.phys_swizzle_mode == arg.swizzle_mode);
 }
@@ -1029,6 +1084,25 @@ run_basic_modes(const struct access_mode *mode,
 					      p->copy, h->hang);
 			}
 
+			/* split copying between rings */
+			igt_subtest_f("%s-%s-write-read-bcs%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				buffers_create(&buffers, num_buffers);
+				run_wrap_func(&buffers,
+					      do_write_read_bcs,
+					      p->copy, h->hang);
+			}
+			igt_subtest_f("%s-%s-write-read-rcs%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				igt_require(rendercopy);
+				buffers_create(&buffers, num_buffers);
+				run_wrap_func(&buffers,
+					      do_write_read_rcs,
+					      p->copy, h->hang);
+			}
+
 			/* and finally try to trick the kernel into loosing the pending write */
 			igt_subtest_f("%s-%s-gpu-read-after-write%s%s", mode->name, p->prefix, suffix, h->suffix) {
 				h->require();
