@@ -27,6 +27,7 @@
 
 #define _GNU_SOURCE
 
+#include "igt.h"
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <stdio.h>
@@ -37,11 +38,8 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_aux.h"
+
+IGT_TEST_DESCRIPTION("Test execbuf fence accounting.");
 
 #define WIDTH 1024
 #define HEIGHT 1024
@@ -85,22 +83,20 @@ static void emit_dummy_load(void)
 	}
 
 	for (i = 0; i < 5; i++) {
-		BLIT_COPY_BATCH_START(devid, tile_flags);
+		BLIT_COPY_BATCH_START(tile_flags);
 		OUT_BATCH((3 << 24) | /* 32 bits */
 			  (0xcc << 16) | /* copy ROP */
 			  pitch);
 		OUT_BATCH(0 << 16 | 1024);
 		OUT_BATCH((2048) << 16 | (2048));
 		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-		BLIT_RELOC_UDW(devid);
 		OUT_BATCH(0 << 16 | 0);
 		OUT_BATCH(pitch);
 		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
-		BLIT_RELOC_UDW(devid);
 		ADVANCE_BATCH();
 
-		if (IS_GEN6(devid) || IS_GEN7(devid)) {
-			BEGIN_BATCH(3);
+		if (batch->gen >= 6) {
+			BEGIN_BATCH(3, 0);
 			OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
 			OUT_BATCH(0);
 			OUT_BATCH(0);
@@ -145,6 +141,14 @@ static void fill_reloc(struct drm_i915_gem_relocation_entry *reloc, uint32_t han
 	reloc->write_domain = 0;
 }
 
+static int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *eb)
+{
+	int err = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, eb))
+		err = errno;
+	return err;
+}
+
 #define BUSY_LOAD (1 << 0)
 #define INTERRUPTIBLE (1 << 1)
 
@@ -193,28 +197,27 @@ static void run_test(int fd, int num_fences, int expected_errno,
 	}
 
 	do {
-		int ret;
-
 		if (flags & BUSY_LOAD)
 			emit_dummy_load();
 
-		ret = drmIoctl(fd,
-			       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-			       &execbuf[0]);
-		igt_assert(expected_errno ?
-		       ret < 0 && errno == expected_errno :
-		       ret == 0);
-
-		ret = drmIoctl(fd,
-			       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-			       &execbuf[1]);
-		igt_assert(expected_errno ?
-		       ret < 0 && errno == expected_errno :
-		       ret == 0);
+		igt_assert_eq(__gem_execbuf(fd, &execbuf[0]), expected_errno);
+		igt_assert_eq(__gem_execbuf(fd, &execbuf[1]), expected_errno);
 	} while (--loop);
 
 	if (flags & INTERRUPTIBLE)
 		igt_stop_signal_helper();
+
+	/* Cleanup */
+	for (n = 0; n < 2*num_fences; n++)
+		gem_close(fd, exec[0][n].handle);
+
+	for (i = 0; i < 2; i++)
+		gem_close(fd, exec[i][2*num_fences].handle);
+
+	if (flags & BUSY_LOAD) {
+		intel_batchbuffer_free(batch);
+		drm_intel_bufmgr_destroy(bufmgr);
+	}
 }
 
 int fd;
@@ -225,7 +228,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 		num_fences = gem_available_fences(fd);
 		igt_assert(num_fences > 4);
 		devid = intel_get_drm_devid(fd);
