@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -39,9 +40,6 @@
 #include <sys/wait.h>
 
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
 
 static const uint32_t canary = 0xdeadbeef;
 
@@ -142,15 +140,16 @@ static void run(data_t *data, int child)
 	 * set-to-gtt-domain within the fault handler.
 	 */
 	if (write) {
-		ptr = gem_mmap(data->fd, handle, size, PROT_READ | PROT_WRITE);
+		ptr = gem_mmap__gtt(data->fd, handle, size,
+				    PROT_READ | PROT_WRITE);
 		ptr[rand() % (size / 4)] = canary;
-	} else
-		ptr = gem_mmap(data->fd, handle, size, PROT_READ);
+	} else {
+		ptr = gem_mmap__gtt(data->fd, handle, size, PROT_READ);
+	}
 	x = ptr[rand() % (size / 4)];
 	munmap(ptr, size);
 
-	igt_assert(x == canary);
-	exit(0);
+	igt_assert_eq_u32(x, canary);
 }
 
 igt_simple_main
@@ -158,35 +157,23 @@ igt_simple_main
 	struct timeval start, end;
 	pid_t children[64];
 	data_t data = {};
-	int n;
 
 	/* check for an intel gpu before goint nuts. */
-	int fd = drm_open_any();
+	int fd = drm_open_driver(DRIVER_INTEL);
 	close(fd);
 
 	igt_skip_on_simulation();
 
-	data.fd = drm_open_any();
+	data.fd = drm_open_driver(DRIVER_INTEL);
 	data.devid = intel_get_drm_devid(data.fd);
 	data.intel_gen = intel_gen(data.devid);
 
 	gettimeofday(&start, NULL);
-	for (n = 0; n < ARRAY_SIZE(children); n++) {
-		switch ((children[n] = fork())) {
-		case -1: igt_assert(0);
-		case 0: run(&data, n); break;
-		default: break;
-		}
-	}
-
-	for (n = 0; n < ARRAY_SIZE(children); n++) {
-		int status = -1;
-		while (waitpid(children[n], &status, 0) == -1 &&
-		       errno == -EINTR)
-			;
-		igt_assert(status == 0);
-	}
+	igt_fork(child, ARRAY_SIZE(children))
+		run(&data, child);
+	igt_waitchildren();
 	gettimeofday(&end, NULL);
+
 	igt_info("Time to execute %lu children:		%7.3fms\n",
 		 ARRAY_SIZE(children), elapsed(&start, &end) / 1000);
 }
