@@ -26,6 +26,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,13 +38,15 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "igt_debugfs.h"
+
+#ifndef PAGE_SIZE
+#define PAGE_SIZE 4096
+#endif
 
 static int OBJECT_SIZE = 16*1024*1024;
 
-static void set_domain(int fd, uint32_t handle)
+static void
+set_domain_gtt(int fd, uint32_t handle)
 {
 	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 }
@@ -53,8 +56,7 @@ mmap_bo(int fd, uint32_t handle)
 {
 	void *ptr;
 
-	ptr = gem_mmap(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
-	igt_assert(ptr != MAP_FAILED);
+	ptr = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
 
 	return ptr;
 }
@@ -84,13 +86,11 @@ test_access(int fd)
 	handle = gem_create(fd, OBJECT_SIZE);
 	igt_assert(handle);
 
-	fd2 = drm_open_any();
+	fd2 = drm_open_driver(DRIVER_INTEL);
 
 	/* Check that fd1 can mmap. */
 	mmap_arg.handle = handle;
-	igt_assert(drmIoctl(fd,
-			    DRM_IOCTL_I915_GEM_MMAP_GTT,
-			    &mmap_arg) == 0);
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg);
 
 	igt_assert(mmap64(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
 			  MAP_SHARED, fd, mmap_arg.offset));
@@ -120,27 +120,26 @@ test_short(int fd)
 	mmap_arg.handle = gem_create(fd, OBJECT_SIZE);
 	igt_assert(mmap_arg.handle);
 
-	igt_assert(drmIoctl(fd,
-			    DRM_IOCTL_I915_GEM_MMAP_GTT,
-			    &mmap_arg) == 0);
-	for (pages = 1; pages <= OBJECT_SIZE / 4096; pages <<= 1) {
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg);
+	for (pages = 1; pages <= OBJECT_SIZE / PAGE_SIZE; pages <<= 1) {
 		uint8_t *r, *w;
 
-		w = mmap64(0, pages * 4096, PROT_READ | PROT_WRITE,
+		w = mmap64(0, pages * PAGE_SIZE, PROT_READ | PROT_WRITE,
 			   MAP_SHARED, fd, mmap_arg.offset);
 		igt_assert(w != MAP_FAILED);
 
-		r = mmap64(0, pages * 4096, PROT_READ,
+		r = mmap64(0, pages * PAGE_SIZE, PROT_READ,
 			   MAP_SHARED, fd, mmap_arg.offset);
 		igt_assert(r != MAP_FAILED);
 
 		for (p = 0; p < pages; p++) {
-			w[4096*p] = r[4096*p];
-			w[4096*p+4095] = r[4096*p+4095];
+			w[p*PAGE_SIZE] = r[p*PAGE_SIZE];
+			w[p*PAGE_SIZE+(PAGE_SIZE-1)] =
+				r[p*PAGE_SIZE+(PAGE_SIZE-1)];
 		}
 
-		munmap(r, pages * 4096);
-		munmap(w, pages * 4096);
+		munmap(r, pages * PAGE_SIZE);
+		munmap(w, pages * PAGE_SIZE);
 	}
 	gem_close(fd, mmap_arg.handle);
 }
@@ -175,8 +174,7 @@ test_read_write(int fd, enum test_read_write order)
 
 	handle = gem_create(fd, OBJECT_SIZE);
 
-	ptr = gem_mmap(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
-	igt_assert(ptr != MAP_FAILED);
+	ptr = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
 
 	if (order == READ_BEFORE_WRITE) {
 		val = *(uint32_t *)ptr;
@@ -199,11 +197,9 @@ test_read_write2(int fd, enum test_read_write order)
 
 	handle = gem_create(fd, OBJECT_SIZE);
 
-	r = gem_mmap(fd, handle, OBJECT_SIZE, PROT_READ);
-	igt_assert(r != MAP_FAILED);
+	r = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ);
 
-	w = gem_mmap(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
-	igt_assert(w != MAP_FAILED);
+	w = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
 
 	if (order == READ_BEFORE_WRITE) {
 		val = *(uint32_t *)r;
@@ -245,7 +241,7 @@ test_write_gtt(int fd)
 
 	/* prefault object into gtt */
 	dst_gtt = mmap_bo(fd, dst);
-	set_domain(fd, dst);
+	set_domain_gtt(fd, dst);
 	memset(dst_gtt, 0, OBJECT_SIZE);
 	munmap(dst_gtt, OBJECT_SIZE);
 
@@ -257,6 +253,155 @@ test_write_gtt(int fd)
 	munmap(src, OBJECT_SIZE);
 }
 
+static void
+test_huge_bo(int fd, int huge, int tiling)
+{
+	uint32_t bo;
+	char *ptr;
+	char *tiled_pattern;
+	char *linear_pattern;
+	uint64_t size, last_offset;
+	int pitch = tiling == I915_TILING_Y ? 128 : 512;
+	int i;
+
+	switch (huge) {
+	case -1:
+		size = gem_mappable_aperture_size() / 2;
+		break;
+	case 0:
+		size = gem_mappable_aperture_size() + PAGE_SIZE;
+		break;
+	default:
+		size = gem_aperture_size(fd) + PAGE_SIZE;
+		break;
+	}
+	intel_require_memory(1, size, CHECK_RAM);
+
+	last_offset = size - PAGE_SIZE;
+
+	/* Create pattern */
+	bo = gem_create(fd, PAGE_SIZE);
+	if (tiling)
+		igt_require(__gem_set_tiling(fd, bo, tiling, pitch) == 0);
+	linear_pattern = gem_mmap__gtt(fd, bo, PAGE_SIZE,
+				       PROT_READ | PROT_WRITE);
+	for (i = 0; i < PAGE_SIZE; i++)
+		linear_pattern[i] = i;
+	tiled_pattern = gem_mmap__cpu(fd, bo, 0, PAGE_SIZE, PROT_READ);
+
+	gem_set_domain(fd, bo, I915_GEM_DOMAIN_CPU | I915_GEM_DOMAIN_GTT, 0);
+	gem_close(fd, bo);
+
+	bo = gem_create(fd, size);
+	if (tiling)
+		igt_require(__gem_set_tiling(fd, bo, tiling, pitch) == 0);
+
+	/* Initialise first/last page through CPU mmap */
+	ptr = gem_mmap__cpu(fd, bo, 0, size, PROT_READ | PROT_WRITE);
+	memcpy(ptr, tiled_pattern, PAGE_SIZE);
+	memcpy(ptr + last_offset, tiled_pattern, PAGE_SIZE);
+	munmap(ptr, size);
+
+	/* Obtain mapping for the object through GTT. */
+	ptr = __gem_mmap__gtt(fd, bo, size, PROT_READ | PROT_WRITE);
+	igt_require_f(ptr, "Huge BO GTT mapping not supported.\n");
+
+	set_domain_gtt(fd, bo);
+
+	/* Access through GTT should still provide the CPU written values. */
+	igt_assert(memcmp(ptr              , linear_pattern, PAGE_SIZE) == 0);
+	igt_assert(memcmp(ptr + last_offset, linear_pattern, PAGE_SIZE) == 0);
+
+	gem_set_tiling(fd, bo, I915_TILING_NONE, 0);
+
+	igt_assert(memcmp(ptr              , tiled_pattern, PAGE_SIZE) == 0);
+	igt_assert(memcmp(ptr + last_offset, tiled_pattern, PAGE_SIZE) == 0);
+
+	munmap(ptr, size);
+
+	gem_close(fd, bo);
+	munmap(tiled_pattern, PAGE_SIZE);
+	munmap(linear_pattern, PAGE_SIZE);
+}
+
+static void
+test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
+{
+	uint64_t huge_object_size, i;
+	uint32_t bo, *pattern_a, *pattern_b;
+	char *a, *b;
+
+	switch (huge) {
+	case -2:
+		huge_object_size = gem_mappable_aperture_size() / 4;
+		break;
+	case -1:
+		huge_object_size = gem_mappable_aperture_size() / 2;
+		break;
+	case 0:
+		huge_object_size = gem_mappable_aperture_size() + PAGE_SIZE;
+		break;
+	default:
+		huge_object_size = gem_aperture_size(fd) + PAGE_SIZE;
+		break;
+	}
+	intel_require_memory(2, huge_object_size, CHECK_RAM);
+
+	pattern_a = malloc(PAGE_SIZE);
+	for (i = 0; i < PAGE_SIZE/4; i++)
+		pattern_a[i] = i;
+
+	pattern_b = malloc(PAGE_SIZE);
+	for (i = 0; i < PAGE_SIZE/4; i++)
+		pattern_b[i] = ~i;
+
+	bo = gem_create(fd, huge_object_size);
+	if (tiling_a)
+		igt_require(__gem_set_tiling(fd, bo, tiling_a, tiling_a == I915_TILING_Y ? 128 : 512) == 0);
+	a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+	igt_require(a);
+	gem_close(fd, bo);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++)
+		memcpy(a + PAGE_SIZE*i, pattern_a, PAGE_SIZE);
+
+	bo = gem_create(fd, huge_object_size);
+	if (tiling_b)
+		igt_require(__gem_set_tiling(fd, bo, tiling_b, tiling_b == I915_TILING_Y ? 128 : 512) == 0);
+	b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+	igt_require(b);
+	gem_close(fd, bo);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++)
+		memcpy(b + PAGE_SIZE*i, pattern_b, PAGE_SIZE);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		if (i & 1)
+			memcpy(a + i *PAGE_SIZE, b + i*PAGE_SIZE, PAGE_SIZE);
+		else
+			memcpy(b + i *PAGE_SIZE, a + i*PAGE_SIZE, PAGE_SIZE);
+	}
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		if (i & 1)
+			igt_assert(memcmp(pattern_b, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
+		else
+			igt_assert(memcmp(pattern_a, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
+	}
+	munmap(a, huge_object_size);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		if (i & 1)
+			igt_assert(memcmp(pattern_b, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
+		else
+			igt_assert(memcmp(pattern_a, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
+	}
+	munmap(b, huge_object_size);
+
+	free(pattern_a);
+	free(pattern_b);
+}
+
 static void
 test_read(int fd)
 {
@@ -283,11 +428,9 @@ test_write_cpu_read_gtt(int fd)
 
 	handle = gem_create(fd, OBJECT_SIZE);
 
-	dst = gem_mmap(fd, handle, OBJECT_SIZE, PROT_READ);
-	igt_assert(dst != (uint32_t *)MAP_FAILED);
+	dst = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ);
 
 	src = gem_mmap__cpu(fd, handle, 0, OBJECT_SIZE, PROT_WRITE);
-	igt_assert(src != (uint32_t *)MAP_FAILED);
 
 	gem_close(fd, handle);
 
@@ -363,39 +506,77 @@ igt_main
 		OBJECT_SIZE = 1 * 1024 * 1024;
 
 	igt_fixture
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
-	igt_subtest("access")
+	igt_subtest("basic")
 		test_access(fd);
-	igt_subtest("short")
+	igt_subtest("basic-short")
 		test_short(fd);
-	igt_subtest("copy")
+	igt_subtest("basic-copy")
 		test_copy(fd);
-	igt_subtest("read")
+	igt_subtest("basic-read")
 		test_read(fd);
-	igt_subtest("write")
+	igt_subtest("basic-write")
 		test_write(fd);
-	igt_subtest("write-gtt")
+	igt_subtest("basic-write-gtt")
 		test_write_gtt(fd);
-	igt_subtest("read-write")
+	igt_subtest("basic-read-write")
 		test_read_write(fd, READ_BEFORE_WRITE);
-	igt_subtest("write-read")
+	igt_subtest("basic-write-read")
 		test_read_write(fd, READ_AFTER_WRITE);
-	igt_subtest("read-write-distinct")
+	igt_subtest("basic-read-write-distinct")
 		test_read_write2(fd, READ_BEFORE_WRITE);
-	igt_subtest("write-read-distinct")
+	igt_subtest("basic-write-read-distinct")
 		test_read_write2(fd, READ_AFTER_WRITE);
 	igt_subtest("fault-concurrent")
 		test_fault_concurrent(fd);
-	igt_subtest("read-no-prefault")
+	igt_subtest("basic-read-no-prefault")
 		run_without_prefault(fd, test_read);
-	igt_subtest("write-no-prefault")
+	igt_subtest("basic-write-no-prefault")
 		run_without_prefault(fd, test_write);
-	igt_subtest("write-gtt-no-prefault")
+	igt_subtest("basic-write-gtt-no-prefault")
 		run_without_prefault(fd, test_write_gtt);
-	igt_subtest("write-cpu-read-gtt")
+	igt_subtest("basic-write-cpu-read-gtt")
 		test_write_cpu_read_gtt(fd);
 
+	igt_subtest("basic-small-bo")
+		test_huge_bo(fd, -1, I915_TILING_NONE);
+	igt_subtest("basic-small-bo-tiledX")
+		test_huge_bo(fd, -1, I915_TILING_X);
+	igt_subtest("basic-small-bo-tiledY")
+		test_huge_bo(fd, -1, I915_TILING_Y);
+
+	igt_subtest("big-bo")
+		test_huge_bo(fd, 0, I915_TILING_NONE);
+	igt_subtest("big-bo-tiledX")
+		test_huge_bo(fd, 0, I915_TILING_X);
+	igt_subtest("big-bo-tiledY")
+		test_huge_bo(fd, 0, I915_TILING_Y);
+
+	igt_subtest("huge-bo")
+		test_huge_bo(fd, 1, I915_TILING_NONE);
+	igt_subtest("huge-bo-tiledX")
+		test_huge_bo(fd, 1, I915_TILING_X);
+	igt_subtest("huge-bo-tiledY")
+		test_huge_bo(fd, 1, I915_TILING_Y);
+
+	igt_subtest("basic-small-copy")
+		test_huge_copy(fd, -2, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("basic-small-copy-XY")
+		test_huge_copy(fd, -2, I915_TILING_X, I915_TILING_Y);
+	igt_subtest("medium-copy")
+		test_huge_copy(fd, -1, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("medium-copy-XY")
+		test_huge_copy(fd, -1, I915_TILING_X, I915_TILING_Y);
+	igt_subtest("big-copy")
+		test_huge_copy(fd, 0, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("big-copy-XY")
+		test_huge_copy(fd, 0, I915_TILING_X, I915_TILING_Y);
+	igt_subtest("huge-copy")
+		test_huge_copy(fd, 1, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("huge-copy-XY")
+		test_huge_copy(fd, 1, I915_TILING_X, I915_TILING_Y);
+
 	igt_fixture
 		close(fd);
 }
