@@ -221,9 +221,8 @@ paint_color_key(struct igt_fb *fb_info)
 	int i, j;
 	uint32_t *fb_ptr;
 
-	fb_ptr = gem_mmap(drm_fd, fb_info->gem_handle,
-			  fb_info->size, PROT_READ | PROT_WRITE);
-	igt_assert(fb_ptr);
+	fb_ptr = gem_mmap__gtt(drm_fd, fb_info->gem_handle, fb_info->size,
+			       PROT_READ | PROT_WRITE);
 
 	for (i = crtc_y; i < crtc_y + crtc_h; i++)
 		for (j = crtc_x; j < crtc_x + crtc_w; j++) {
@@ -364,6 +363,7 @@ set_mode(struct connector *c)
 		if (drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,
 				   &c->id, 1, &c->mode)) {
 			igt_warn("failed to set mode (%dx%d@%dHz): %s\n", width, height, c->mode.vrefresh, strerror(errno));
+			igt_remove_fb(drm_fd, &fb_info[current_fb]);
 			continue;
 		}
 
@@ -387,7 +387,7 @@ set_mode(struct connector *c)
 		}
 	}
 
-	if (test_all_modes)
+	if (test_all_modes && old_fb != -1)
 		igt_remove_fb(drm_fd, &fb_info[old_fb]);
 
 	drmModeFreeEncoder(c->encoder);
@@ -552,7 +552,7 @@ static void __attribute__((noreturn)) usage(char *name, char opt)
 
 #define dump_resource(res) if (res) dump_##res()
 
-static void cleanup_and_exit(int ret)
+static void __attribute__((noreturn)) cleanup_and_exit(int ret)
 {
 	close(drm_fd);
 	exit(ret);
