@@ -39,6 +39,7 @@
  */
 
 #define _GNU_SOURCE
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -52,8 +53,10 @@
 #include <intel_bufmgr.h>
 
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "instdone.h"
+#include "intel_reg.h"
+#include "drmtest.h"
 
 static uint32_t
 print_head(unsigned int reg)
@@ -94,14 +97,15 @@ print_instdone(uint32_t devid, unsigned int instdone, unsigned int instdone1)
 	static int once;
 
 	if (!once) {
-		init_instdone_definitions(devid);
+		if (!init_instdone_definitions(devid))
+			return;
 		once = 1;
 	}
 
 	for (i = 0; i < num_instdone_bits; i++) {
 		int busy = 0;
 
-		if (instdone_bits[i].reg == INST_DONE_1) {
+		if (instdone_bits[i].reg == INSTDONE_1) {
 			if (!(instdone1 & instdone_bits[i].bit))
 				busy = 1;
 		} else {
@@ -241,6 +245,78 @@ print_pgtbl_err(unsigned int reg, unsigned int devid)
 	}
 }
 
+static void print_ivb_error(unsigned int reg, unsigned int devid)
+{
+	if (reg & (1 << 0))
+		printf("    TLB page fault error (GTT entry not valid)\n");
+	if (reg & (1 << 1))
+		printf("    Invalid physical address in RSTRM interface (PAVP)\n");
+	if (reg & (1 << 2))
+		printf("    Invalid page directory entry error\n");
+	if (reg & (1 << 3))
+		printf("    Invalid physical address in ROSTRM interface (PAVP)\n");
+	if (reg & (1 << 4))
+		printf("    TLB page VTD translation generated an error\n");
+	if (reg & (1 << 5))
+		printf("    Invalid physical address in WRITE interface (PAVP)\n");
+	if (reg & (1 << 6))
+		printf("    Page directory VTD translation generated error\n");
+	if (reg & (1 << 8))
+		printf("    Cacheline containing a PD was marked as invalid\n");
+	if (IS_HASWELL(devid) && (reg >> 10) & 0x1f)
+		printf("    %d pending page faults\n", (reg >> 10) & 0x1f);
+}
+
+static void print_snb_error(unsigned int reg)
+{
+	if (reg & (1 << 0))
+		printf("    TLB page fault error (GTT entry not valid)\n");
+	if (reg & (1 << 1))
+		printf("    Context page GTT translation generated a fault (GTT entry not valid)\n");
+	if (reg & (1 << 2))
+		printf("    Invalid page directory entry error\n");
+	if (reg & (1 << 3))
+		printf("    HWS page GTT translation generated a page fault (GTT entry not valid)\n");
+	if (reg & (1 << 4))
+		printf("    TLB page VTD translation generated an error\n");
+	if (reg & (1 << 5))
+		printf("    Context page VTD translation generated an error\n");
+	if (reg & (1 << 6))
+		printf("    Page directory VTD translation generated error\n");
+	if (reg & (1 << 7))
+		printf("    HWS page VTD translation generated an error\n");
+	if (reg & (1 << 8))
+		printf("    Cacheline containing a PD was marked as invalid\n");
+}
+
+static void print_bdw_error(unsigned int reg, unsigned int devid)
+{
+	print_ivb_error(reg, devid);
+
+	if (reg & (1 << 10))
+		printf("    Non WB memory type for Advanced Context\n");
+	if (reg & (1 << 11))
+		printf("    PASID not enabled\n");
+	if (reg & (1 << 12))
+		printf("    PASID boundary violation\n");
+	if (reg & (1 << 13))
+		printf("    PASID not valid\n");
+	if (reg & (1 << 14))
+		printf("    PASID was zero for untranslated request\n");
+	if (reg & (1 << 15))
+		printf("    Context was not marked as present when doing DMA\n");
+}
+
+static void
+print_error(unsigned int reg, unsigned int devid)
+{
+	switch (intel_gen(devid)) {
+	case 8: return print_bdw_error(reg, devid);
+	case 7: return print_ivb_error(reg, devid);
+	case 6: return print_snb_error(reg);
+	}
+}
+
 static void
 print_snb_fence(unsigned int devid, uint64_t fence)
 {
@@ -305,18 +381,81 @@ print_fence(unsigned int devid, uint64_t fence)
 	}
 }
 
+static void
+print_fault_reg(unsigned devid, uint32_t reg)
+{
+	const char *gen7_types[] = { "Page",
+				     "Invalid PD",
+				     "Unloaded PD",
+				     "Invalid and Unloaded PD" };
+
+	const char *gen8_types[] = { "PTE",
+				     "PDE",
+				     "PDPE",
+				     "PML4E" };
+
+	const char *engine[] = { "GFX", "MFX0", "MFX1", "VEBX",
+				 "BLT", "Unknown", "Unknown", "Unknown" };
+
+	if (intel_gen(devid) < 7)
+		return;
+
+	if (reg & (1 << 0))
+		printf("    Valid\n");
+	else
+		return;
+
+	if (intel_gen(devid) < 8)
+		printf("    %s Fault (%s)\n", gen7_types[reg >> 1 & 0x3],
+		       reg & (1 << 11) ? "GGTT" : "PPGTT");
+	else
+		printf("    Invalid %s Fault\n", gen8_types[reg >> 1 & 0x3]);
+
+	if (intel_gen(devid) < 8)
+		printf("    Address 0x%08x\n", reg & ~((1 << 12)-1));
+	else
+		printf("    Engine %s\n", engine[reg >> 12 & 0x7]);
+
+	printf("    Source ID %d\n", reg >> 3 & 0xff);
+}
+
+static void
+print_fault_data(unsigned devid, uint32_t data1, uint32_t data0)
+{
+	uint64_t address;
+
+	if (intel_gen(devid) < 8)
+		return;
+
+	address = ((uint64_t)(data0) << 12) | ((uint64_t)data1 & 0xf) << 44;
+	printf("    Address 0x%016" PRIx64 " %s\n", address,
+	       data1 & (1 << 4) ? "GGTT" : "PPGTT");
+}
+
 #define MAX_RINGS 10 /* I really hope this never... */
 uint32_t head[MAX_RINGS];
 int head_ndx = 0;
-int num_rings = -1;
+int num_rings = 0;
 static void print_batch(int is_batch, const char *ring_name, uint32_t gtt_offset)
 {
 	const char *buffer_type[2] = {  "ringbuffer", "batchbuffer" };
-	if (is_batch) {
+	if (is_batch || !num_rings)
 		printf("%s (%s) at 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset);
-	} else {
+	else
 		printf("%s (%s) at 0x%08x; HEAD points to: 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset, head[head_ndx++ % num_rings] + gtt_offset);
-	}
+}
+
+static void decode(struct drm_intel_decode *ctx, bool is_batch,
+		   const char *ring_name, uint32_t gtt_offset, uint32_t *data,
+		   int *count)
+{
+	if (!*count)
+		return;
+
+	print_batch(is_batch, ring_name, gtt_offset);
+	drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset, *count);
+	drm_intel_decode(ctx);
+	*count = 0;
 }
 
 static void
@@ -350,14 +489,8 @@ read_data_file(FILE *file)
 			matched = sscanf(dashes, "--- gtt_offset = 0x%08x\n",
 					&new_gtt_offset);
 			if (matched == 1) {
-				if (count) {
-					print_batch(is_batch, ring_name, gtt_offset);
-					drm_intel_decode_set_batch_pointer(decode_ctx,
-							data, gtt_offset,
-							count);
-					drm_intel_decode(decode_ctx);
-					count = 0;
-				}
+				decode(decode_ctx, is_batch, ring_name,
+				       gtt_offset, data, &count);
 				gtt_offset = new_gtt_offset;
 				is_batch = 1;
 				free(ring_name);
@@ -368,14 +501,8 @@ read_data_file(FILE *file)
 			matched = sscanf(dashes, "--- ringbuffer = 0x%08x\n",
 					&new_gtt_offset);
 			if (matched == 1) {
-				if (count) {
-					print_batch(is_batch, ring_name, gtt_offset);
-					drm_intel_decode_set_batch_pointer(decode_ctx,
-							data, gtt_offset,
-							count);
-					drm_intel_decode(decode_ctx);
-					count = 0;
-				}
+				decode(decode_ctx, is_batch, ring_name,
+				       gtt_offset, data, &count);
 				gtt_offset = new_gtt_offset;
 				is_batch = 0;
 				free(ring_name);
@@ -386,17 +513,11 @@ read_data_file(FILE *file)
 
 		matched = sscanf(line, "%08x : %08x", &offset, &value);
 		if (matched != 2) {
-			unsigned int reg;
+			unsigned int reg, reg2;
 
 			/* display reg section is after the ringbuffers, don't mix them */
-			if (count) {
-				print_batch(is_batch, ring_name, gtt_offset);
-				drm_intel_decode_set_batch_pointer(decode_ctx,
-						data, gtt_offset,
-						count);
-				drm_intel_decode(decode_ctx);
-				count = 0;
-			}
+			decode(decode_ctx, is_batch, ring_name, gtt_offset,
+			       data, &count);
 
 			printf("%s", line);
 
@@ -422,7 +543,7 @@ read_data_file(FILE *file)
 
 			matched = sscanf(line, "  HEAD: 0x%08x\n", &reg);
 			if (matched == 1) {
-				head[head_ndx++] = print_head(reg);
+				head[num_rings++] = print_head(reg);
 			}
 
 			matched = sscanf(line, "  ACTHD: 0x%08x\n", &reg);
@@ -435,6 +556,10 @@ read_data_file(FILE *file)
 			if (matched == 1 && reg)
 				print_pgtbl_err(reg, devid);
 
+			matched = sscanf(line, "  ERROR: 0x%08x\n", &reg);
+			if (matched == 1 && reg)
+				print_error(reg, devid);
+
 			matched = sscanf(line, "  INSTDONE: 0x%08x\n", &reg);
 			if (matched == 1)
 				print_instdone(devid, reg, -1);
@@ -447,6 +572,14 @@ read_data_file(FILE *file)
 			if (matched == 2)
 				print_fence(devid, fence);
 
+			matched = sscanf(line, "  FAULT_REG: 0x%08x\n", &reg);
+			if (matched == 1 && reg)
+				print_fault_reg(devid, reg);
+
+			matched = sscanf(line, "  FAULT_TLB_DATA: 0x%08x 0x%08x\n", &reg, &reg2);
+			if (matched == 2)
+				print_fault_data(devid, reg, reg2);
+
 			continue;
 		}
 
@@ -464,13 +597,7 @@ read_data_file(FILE *file)
 		data[count-1] = value;
 	}
 
-	if (count) {
-		print_batch(is_batch, ring_name, gtt_offset);
-		drm_intel_decode_set_batch_pointer(decode_ctx,
-				data, gtt_offset,
-				count);
-		drm_intel_decode(decode_ctx);
-	}
+	decode(decode_ctx, is_batch, ring_name, gtt_offset, data, &count);
 
 	free(data);
 	free(line);
